/*
sdlacfg.h

	Property page provider for sdladrv.sys on Windows 2000/XP/2003.

	Author: David Rokhvarg
*/

#ifndef _SDLACFG_H
#define _SDLACFG_H

#include "resource.h"

#include <windowsx.h>			//for GET_WM_COMMAND_CMD() macro
#include <stdio.h>				//for snprintf()
#include <stdlib.h>				//for atoi()

#include "wanpipe_defines.h"	//for 'wan_udp_hdr_t'
#include "sang_status_defines.h"//return codes
#include "sang_api.h"			//for IOCTL codes
#include "wanpipe_cfg.h"		//for WAN_MEDIA_NONE...
#include "sdlapci.h"			//for AFT_8TE1_SHARK_SUBSYS_VENDOR...
#include "bit_win.h"

//define this to get output in WinDbg
//#define _MYDEBUG

#define PROGRAM_NAME "SdlaCfg: "
#include "DebugOut.h"	//debug macros

#define snprintf _snprintf
//to get debug output from a page define as 1
#define DRV_CFG_PAGE	0
#define DBG_SAM_CLASS	0
#define DBG_AFT_PAGE	0	
#define DBG_AFTUP		0
#define DBG_ANALOG_PAGE	0
#define DBG_AFT_SERIAL_PAGE	0
#define DBG_ADSL_PAGE	0
#define DBG_AFT_BRI_PAGE	0

//Help topics IDs
#define IDH_DRIVER_CFG		1
#define IDH_LINE_CFG		4

#ifndef __FUNCTION__
#define __FUNCTION__ "function"
#endif

////////////////////////////////////////////////////////////////////////////////

DLGPROC_RETURN_TYPE WINAPI AFT_T1E1_PageDlgProc(HWND hdlg, UINT msg, WPARAM wParam, LPARAM lParam);
UINT CALLBACK AFT_T1E1_PageCallbackProc(HWND junk, UINT msg, LPPROPSHEETPAGE p);

DLGPROC_RETURN_TYPE WINAPI DrvPageDlgProc(HWND hdlg, UINT msg, WPARAM wParam, LPARAM lParam);
UINT CALLBACK DrvPageCallbackProc(HWND junk, UINT msg, LPPROPSHEETPAGE p);

DLGPROC_RETURN_TYPE WINAPI AFT_PageDlgProc(HWND hdlg, UINT msg, WPARAM wParam, LPARAM lParam);
UINT CALLBACK AFT_PageCallbackProc(HWND junk, UINT msg, LPPROPSHEETPAGE p);

DLGPROC_RETURN_TYPE WINAPI AFTFirmwareUpdateDlgProc(HWND hdlg, UINT msg, WPARAM wParam, LPARAM lParam);
UINT CALLBACK AFTFirmwareUpdatePageCallbackProc(HWND junk, UINT msg, LPPROPSHEETPAGE p);

DLGPROC_RETURN_TYPE WINAPI AFT_Analog_PageDlgProc(HWND hdlg, UINT msg, WPARAM wParam, LPARAM lParam);
UINT CALLBACK AFT_Analog_PageCallbackProc(HWND junk, UINT msg, LPPROPSHEETPAGE p);

DLGPROC_RETURN_TYPE WINAPI AFT_Serial_PageDlgProc(HWND hdlg, UINT msg, WPARAM wParam, LPARAM lParam);
UINT CALLBACK AFT_Serial_PageCallbackProc(HWND junk, UINT msg, LPPROPSHEETPAGE p);

DLGPROC_RETURN_TYPE WINAPI ADSL_PageDlgProc(HWND hdlg, UINT msg, WPARAM wParam, LPARAM lParam);
UINT CALLBACK ADSL_PageCallbackProc(HWND junk, UINT msg, LPPROPSHEETPAGE p);

DLGPROC_RETURN_TYPE WINAPI AFT_BRI_PageDlgProc(HWND hdlg, UINT msg, WPARAM wParam, LPARAM lParam);
UINT CALLBACK AFT_BRI_PageCallbackProc(HWND junk, UINT msg, LPPROPSHEETPAGE p);

////////////////////////////////////////////////////////////////////////////////

DWORD GetRegistryDWordValue(HKEY hKey, LPTSTR szKeyname);
void SetRegistryDWordValue(HKEY hKey, LPTSTR szKeyname, DWORD dwValue);
void GetRegistryStringValue(HKEY hkey, LPTSTR szKeyname, LPSTR szValue, DWORD * pdwSize);
void SetRegistryStringValue(HKEY hkey, LPTSTR szKeyname, LPSTR szValue);
void SetRegistryStringValue(HKEY hkey, LPTSTR szKeyname, int iValue);

//returns the number in char array or -1 if not a number
int		is_number(char * buff);
BOOL	CheckActiveChannels(PUCHAR ActiveChannels, unsigned char media_type);
unsigned long parse_active_channel(char* val, unsigned char media_type, unsigned char is_unframed_e1);
int		yes_no_question(char* question);
void	restart_device();
int		start_monitor(int if_index);
void	set_dialog_wanpipe_name(HWND hdlg);
bool is_legacy_api_mode(char *operational_mode);

#define NO FALSE
#define YES TRUE

#define BIGBUFF 256

//return codes from OnNotfy()
#define HANDLE_DEFAULT	0
#define HANDLE_OK		1
#define HANDLE_ERROR	2


typedef struct _look_up
{
	const char	*name;
	u32			value;
}look_up_t;


//This is defined in wanpipe_tdm_api.h, but only for kernel,
//so have to redefine here.
#define WP_TDM_API_MAX_LEN 	1024

#define MIN_AFT_MTU		8	
#define MAX_AFT_MTU		8188

#define TMP_BUFFER_LEN	512

struct SETUPSTUFF {
	HDEVINFO info;
	PSP_DEVINFO_DATA did;
};

#define NOT_INIT_STR	"?"

typedef struct _AFT_LOGIC_CH_CFG{

	char active_ch_str[TMP_BUFFER_LEN];
	char line_mode[TMP_BUFFER_LEN];
	char mtu[TMP_BUFFER_LEN];
	char operational_mode[TMP_BUFFER_LEN];
	unsigned char decimal_idle_char;
	unsigned char analog_module_type;	//MOD_TYPE_FXO or MOD_TYPE_FXS

}AFT_LOGIC_CH_CFG;


typedef struct _GLOBALS
{
	int log_errors;

	char DeviceName[TMP_BUFFER_LEN];
#define MIN_WP_NUMBER	0	/* 0 means auto, else user provided */
#define MAX_WP_NUMBER	128
	unsigned short	wanpipe_number;

	wandev_conf_t	wandev_conf;
	char dchan_active_ch_str[TMP_BUFFER_LEN];

	//AFT
	unsigned short aft_number_logic_channels;
	AFT_LOGIC_CH_CFG aft_logic_ch_cfg[NUM_OF_E1_TIMESLOTS];

	//ADSL
	wan_adsl_conf_t	adsl_cfg;

	//Serial card parameters - FIXME: consider movind to 'wandev_conf' above, it all should be there.
	u32 serial_connection_type, serial_line_coding, serial_line_idle;

	u32 start_port_when_computer_starts;

	//BRI
	char BriModuleType[TMP_BUFFER_LEN];
}GLOBALS;

extern GLOBALS Globals_Old_Cfg;
extern GLOBALS Globals_New_Cfg;
extern SETUPSTUFF * stuff;
extern HKEY	hkey;
extern BOOL	okay;
extern UCHAR released_by_OK_button;
extern HINSTANCE hInst;
extern int iCardType;
extern int iCardSubType;
extern unsigned int iAutoWanpipeNumber;

//////////////////////////////////////////////////////////////////////
//ComboBox Functions
//////////////////////////////////////////////////////////////////////
static
void
cfg_combobox_init(
	IN HWND hdlg,
	IN int	combo_box_id,
	IN const look_up_t *cfg_array,
	IN int	value
	)
{
	int n, ind;

	//start from removing everything from the combobox
	SendDlgItemMessage(hdlg, combo_box_id, CB_RESETCONTENT, 0, 0);

	//add the lines to combobox
	n = 0;
	while(cfg_array[n].name != NULL){
		SendDlgItemMessage(	hdlg, combo_box_id, CB_ADDSTRING, 0, (LPARAM)cfg_array[n].name);
		n++;
	}

	//find index of line which should be selected based on the current value
	n = 0; ind = 0;
	while(cfg_array[n].name != NULL){
		if((int)cfg_array[n].value == value){
			ind = n; 
			break;
		}
		n++;
	}

	//select the line in the combobox
	SendDlgItemMessage(	hdlg, combo_box_id, CB_SETCURSEL, ind, 0);
}

static 
int 
cfg_combobox_get_value(
	IN HWND hdlg,
	IN int	combo_box_id,
	IN const look_up_t *cfg_array
	)
{
	int n, ind;

	//get index of current selection
	ind = SendDlgItemMessage(hdlg, combo_box_id, CB_GETCURSEL, 0, 0);

	//find the corresponding value of line in combobox
	n = 0;
	while(cfg_array[n].name != NULL){
		if(n == ind){
			return cfg_array[ind].value;
		}
		n++;
	}

	return 0;
}

//////////////////////////////////////////////////////////////////////
//TextBox Functions
//////////////////////////////////////////////////////////////////////
static
void
CfgTextBoxSetIntegerValue(
	IN HWND hdlg,
	IN int	text_box_id,
	IN int	value
	)
{
	char local_tmp_buff[TMP_BUFFER_LEN];

	snprintf(local_tmp_buff, sizeof(local_tmp_buff), "%d", value);
	SetDlgItemText(hdlg, text_box_id, local_tmp_buff);
}

/* This function allows to get both Negative and Positive 
 * numeric values from Edit Box. */
static 
int 
CfgTextBoxGetIntegerValue(
	IN HWND hdlg,
	IN int	text_box_id
	)
{
    BOOL bTranslated;
	UINT result;

	result = GetDlgItemInt(hdlg, text_box_id, &bTranslated, TRUE);

	if(bTranslated == TRUE){
		//ok
		return (INT)result;
	}else{
		//error
		char local_tmp_buff[TMP_BUFFER_LEN];

		_snprintf(local_tmp_buff, sizeof(local_tmp_buff), "Non-numeric input: ");
		GetDlgItemText(hdlg, text_box_id, 
			&local_tmp_buff[strlen(local_tmp_buff)],
			sizeof(local_tmp_buff) - strlen(local_tmp_buff)
			);

		MessageBox(GetActiveWindow(),
			local_tmp_buff, 
			"Configuration Error",
			MB_OK|MB_ICONWARNING|MB_SYSTEMMODAL
			);
		return 0;
	}
}

static 
int 
CfgTextBoxGetStringValue(
	IN HWND hdlg,
	IN int	text_box_id,
	IN OUT	char *dest_buffer,
	IN int	dest_buffer_size
	)
{
	UINT rc;

	rc = GetDlgItemText(hdlg, text_box_id, dest_buffer, dest_buffer_size);
	if(!rc){
		//error
		return -1;
	}else{
		return 0;
	}
}


#endif //_SDLACFG_H
