/*
 * Copyright (c) 2010
 *	Sangoma Technologies Corporation.  All rights reserved.
 *
 */

/*****************************************************************************
 * sdla_fe_stack_iface.h	Sangoma-to-TMS iterface definitions.
 *
 *	The definitions are shared between open-source and
 *	closed-source parts of the code, and must NOT use
 *	any data types which dependend on version of 
 *	Linux kernel (such as spin_lock, timer...).
 *
 *	Functions starting with "sdla_fe_stack_te1" used only
 *	by TE1 part of Stack.
 *	Functions starting with "sdla_fe_stack" used by both 
 *	TE1 and Analog parts of Stack.
 *
 * Author:      David Rokhvarg	<davidr@sangoma.com>
 *
 * ============================================================================
 *
 * April 14, 2010	David Rokhvarg		Initial version.
 *
 ****************************************************************************
*/
#ifndef	_SDLA_FE_STACK_IFACE_H
#define	_SDLA_FE_STACK_IFACE_H

/******************************************************************************
			  DEFINES AND MACROS
******************************************************************************/

typedef struct _sdladrv_fe_ops {

	/* TE1 Register read/write */
	unsigned char (*sdla_fe_stack_te1_read_fe)(void *phw, int reg);
	int	(*sdla_fe_stack_te1_write_fe)(void *phw, int reg, unsigned char in_value);

	/* Analog Register read/write */
	/* to be implemented */

	/* Common for all parts of Stack */
#if !defined(__WINDOWS__)
	void	(*sdla_fe_stack_log)(unsigned int logger_type, unsigned int evt_type, const char *fmt, ...);
#endif
} sdladrv_fe_ops_t;

#define WANPIPE_FE_MODNAME	"wanpipe_fe"
#define WANPIPE_FE_MODVERSION	"1.0.0"

/******************************************************************************
			  FUNCTION PROTOTYPES
******************************************************************************/

#if defined(__WINDOWS__)
/* functions called by sngbus.sys (implementation in sdladrv_fe.c): */
extern unsigned char _sdla_fe_stack_te1_read_fe(void *phw, int reg);
extern int _sdla_fe_stack_te1_write_fe(void *phw, int reg, unsigned char in_value);
#else
extern unsigned char sdla_fe_stack_te1_read_fe(void *phw, int reg);
extern int sdla_fe_stack_te1_write_fe(void *phw, int reg, unsigned char in_value);
#endif

typedef int	(FE_STACK_ISR_T)(void);

/*****************************************************************************/
/* functions in sdla_ncomm_te1.c */
extern int _sdla_fe_stack_init(sdladrv_fe_ops_t *sdladrv_fe_ops);
extern int _sdla_fe_stack_exit(void);

/* sdla_fe_stack_te1_iface_init() will get pointers to these functions: */
extern char* sdla_fe_stack_te1_get_fe_media_string(void);
extern unsigned char sdla_fe_stack_te1_get_fe_media(void *fe);
		
extern int sdla_fe_stack_te1_global_config(void* pfe);
extern int sdla_fe_stack_te1_global_unconfig(void* pfe);

extern int sdla_fe_stack_te1_global_config(void* pfe);	
extern int sdla_fe_stack_te1_global_unconfig(void* pfe);	

extern int sdla_fe_stack_te1_init(void* pfe);
extern int sdla_fe_stack_te1_post_init(void *pfe);

extern int sdla_fe_stack_te1_post_unconfig(void* pfe);
extern int sdla_fe_stack_te1_unconfig(void* pfe);	

extern int sdla_fe_stack_te1_reconfig(void* fe);
extern int sdla_fe_stack_te1_reset(void* fe, int port_no, int reset);
extern int sdla_fe_stack_te1_chip_config(void* pfe);

extern int sdla_fe_stack_te1_read_pmon_counters(void *fe, sdla_te_pmon_cmd_t *pmon_cmd);
extern unsigned int sdla_fe_stack_te1_read_alarms(void *fe, int action_not_used);

extern int sdla_fe_stack_te1_pmon(void *fe, int action_not_used);
extern int sdla_fe_stack_te1_flush_pmon(void *fe);

/* transmission of LB codes available only on T1 */
extern int sdla_fe_stack_t1_tx_lb(void *fe, unsigned char type, unsigned char mode);
extern int sdla_fe_stack_te1_loopback_ctrl(void *fe, int lb_type, int sangoma_lb_ctrl_code);
extern int sdla_loopback_type_Sangoma2FeStack(int sangoma_lb_type);

extern int sdla_fe_stack_te1_set_lb(void*, unsigned char, unsigned char, u_int32_t); 

extern int sdla_fe_stack_te1_register_address(void *fe, int port_no, int reg);
extern int sdla_fe_stack_te1_intr(void *fe);

extern int sdla_fe_stack_te1_sigctrl(void *fe, int sig_mode, unsigned long ch_map, int mode);
extern int sdla_fe_stack_te1_get_fe_status(void *fe, unsigned char *status, int not_used);			
		
/************ CAS-related functions ************/		
extern int __sdla_fe_stack_set_cas_state(void *fe, int channel, unsigned int fe_stack_signal, char digit);

extern unsigned char sdla_translate_cas_signal_from_FeStack_to_Sangoma(
	void* fe, int channo, unsigned int in_fe_stack_rxbits, unsigned char *out_sangoma_rxbits);

extern unsigned char sdla_translate_cas_signal_from_Sangoma_to_FeStack(
	void* fe, int channo, unsigned char in_sangoma_txbits, unsigned int *out_fe_stack_txbits);

/***********************************************/
		
/*****************************************************************************/
/* functions in wanfe.c which are used by binary part of FE Stack and
 * by code outside of wanpipe_fe.ko module */
extern void wanfe_global_init(void);/* entry point of wanpipe_fe.ko */
extern void wanfe_global_exit(void);/* unload of of wanpipe_fe.ko */

extern int sdla_fe_stack_init(sdladrv_fe_ops_t *sdladrv_fe_ops);
extern int sdla_fe_stack_exit(void);

extern void sdla_fe_stack_set_interrupt_handler(void *fe, FE_STACK_ISR_T *inthandler);
extern FE_STACK_ISR_T* sdla_fe_stack_get_interrupt_handler(void *fe);
extern void* sdla_fe_stack_get_device_base_adr(void *fe);
extern void* get_fe_ptr_for_line_no(int line_no);
extern void sdla_fe_stack_trigger_poll(void *fe);
extern int sdla_fe_stack_get_line_number(void *fe);
extern int sdla_fe_stack_add_fe_to_lines_list(void *fe);
extern int sdla_fe_stack_remove_fe_from_lines_list(void *fe);
extern void sdla_fe_stack_wait_for_unhookinterrupt(void *fe);

/* Raw RBS/CAS */
extern int sdla_fe_stack_te1_store_rx_cas_raw_state(void *fe, int channo, 
		int /* INTENTIONALLY not a char */ fe_stack_rxbits);

/* Signaling Model RBS/CAS */
extern int sdla_fe_stack_rx_cas_signal(void *fe, int channo, 
		int /* INTENTIONALLY not a char */ fe_stack_signal, 
		char *digit_str);
extern int sdla_fe_stack_tx_cas_signal(void *fe, int channo, 
		int /* INTENTIONALLY not a char */ fe_stack_signal, 
		char *digit_str);

extern int sdla_fe_stack_cas_signal_tx_complete(void *fe, int channo);
extern int sdla_fe_stack_cas_signal_tx_overflow(void *fe, int channo);
extern int sdla_fe_stack_cas_signal_tx_invalid(void *fe, int channo);

extern const char* wanfe_pm_interval_decode(wan_pm_interval_t pm_interval);
		
/****************************************************************************/
/*				Read/Write to front-end register							*/
/****************************************************************************/		
int sdla_fe_stack_te1_write(void *BaseAdr /* hw ptr */, int reg, unsigned char in_value);
int sdla_fe_stack_te1_read(void *BaseAdr /* hw ptr */, int reg);
		
#define FE_STACK_TE1_WRITE_REG(reg,val)	\
	sdla_fe_stack_te1_write(	\
		wpabs_fe_get_hw_ptr(fe),	\
		(int)sdla_fe_stack_te1_register_address(fe, wpabs_fe_get_line_no(fe), (reg)),	\
		(int)(val))

#define FE_STACK_TE1_READ_REG(reg)	\
	sdla_fe_stack_te1_read(	\
		wpabs_fe_get_hw_ptr(fe),	\
		(int)sdla_fe_stack_te1_register_address(fe, wpabs_fe_get_line_no(fe), (reg)))
	
#define FE_STACK_TE1_READ_REG_LINE(fe_line_no, reg)	\
	sdla_fe_stack_te1_read(		\
		wpabs_fe_get_hw_ptr(fe),	\
		(int)sdla_fe_stack_te1_register_address(fe, fe_line_no, (reg)))
		
/*****************************************************************************/
		
#endif /* _SDLA_FE_STACK_IFACE_H */
