/*
 * Copyright (c) 2001
 *	Sangoma Technologies.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by Sangoma Technologies.
 * 4. Neither the name of the author nor the names of any co-contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Sangoma Technologies AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL Sangoma Technologies
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 *
 *	$Id: sdla_te1.h,v 1.72 2008-04-02 20:52:09 sangoma Exp $
 */

/*****************************************************************************
 * sdla_te1.h	Sangoma TE1 configuration definitions.
 *
 * Authors:		Alex Feldman
 *				David Rokhvarg
 *
 * ============================================================================
 *
 * Sept 15, 2010	David Rokhvarg	Definitions for T1.231 (T1) and G826 (E1)
 *			Performance Monitoring protocols.
 *
 * Aprl 30, 2001	Alex Feldman	Initial version.
 ****************************************************************************
*/
#ifndef	_SDLA_TE1_H
#    define	_SDLA_TE1_H

#include "aft_core_options.h"

#ifdef SDLA_TE1
# define WP_EXTERN
#else
# define WP_EXTERN extern
#endif

/************************************************************************
 *			  DEFINES AND MACROS				*
 ***********************************************************************/

/*
*************************************************************************
*			  DEFINES AND MACROS				*
*************************************************************************
*/

#define WAN_TE_CHIP_PMC		0x00
#define WAN_TE_CHIP_DM		0x01

#define NUM_OF_T1_CHANNELS	24
#define NUM_OF_E1_TIMESLOTS	31
#define NUM_OF_E1_CHANNELS	32
#define ENABLE_ALL_CHANNELS	0xFFFFFFFF

#define E1_FRAMING_TIMESLOT	0
#define E1_SIGNALING_TIMESLOT	16

#define WAN_TE_SIG_POLL		0x01
#define WAN_TE_SIG_INTR		0x02

/* Framer Alarm bit mask */
#define WAN_TE_ALARM_LIU		0x80000000

#define WAN_TE_ALARM_FRAMER_MASK    	0x0000FFFF
#define WAN_TE_ALARM_LIU_MASK		0x00F00000

#define WAN_TE_BIT_ALARM_ALOS		0x00000001
#define WAN_TE_BIT_ALARM_LOS		0x00000002
#define WAN_TE_BIT_ALARM_ALTLOS		0x00000004
#define WAN_TE_BIT_ALARM_OOF		0x00000008
#define WAN_TE_BIT_ALARM_RED		0x00000010
#define WAN_TE_BIT_ALARM_AIS		0x00000020
#define WAN_TE_BIT_ALARM_OOSMF		0x00000040
#define WAN_TE_BIT_ALARM_OOCMF		0x00000080
#define WAN_TE_BIT_ALARM_OOOF		0x00000100
#define WAN_TE_BIT_ALARM_RAI		0x00000200
#define WAN_TE_BIT_ALARM_YEL		0x00000400
#define WAN_TE_BIT_ALARM_LOF		0x00000800

#define WAN_TE_BIT_LOOPUP_CODE		0x00002000	
#define WAN_TE_BIT_LOOPDOWN_CODE	0x00004000	

#define WAN_TE_BIT_ALARM_LIU		0x00100000
#define WAN_TE_BIT_ALARM_LIU_SC		0x00100000
#define WAN_TE_BIT_ALARM_LIU_OC		0x00200000
#define WAN_TE_BIT_ALARM_LIU_LOS	0x00400000

/* additions for alarms reported by TMS */
#define WAN_TE_BIT_ALARM_RAIC		0x01000000
#define WAN_TE_BIT_ALARM_OOFC		0x02000000

#define IS_TE_ALARM(alarm, mask)	(alarm & mask)
#define IS_TE_ALARM_ALOS(alarm)		IS_TE_ALARM(alarm, WAN_TE_BIT_ALARM_ALOS)
#define IS_TE_ALARM_LOS(alarm)		IS_TE_ALARM(alarm, WAN_TE_BIT_ALARM_LOS)
#define IS_TE_ALARM_OOF(alarm)		IS_TE_ALARM(alarm, WAN_TE_BIT_ALARM_OOF)
#define IS_TE_ALARM_LOF(alarm)		IS_TE_ALARM(alarm, WAN_TE_BIT_ALARM_LOF)
#define IS_TE_ALARM_RED(alarm)		IS_TE_ALARM(alarm, WAN_TE_BIT_ALARM_RED)
#define IS_TE_ALARM_AIS(alarm)		IS_TE_ALARM(alarm, WAN_TE_BIT_ALARM_AIS)
#define IS_TE_ALARM_OOSMF(alarm)	IS_TE_ALARM(alarm, WAN_TE_BIT_ALARM_OOSMF)
#define IS_TE_ALARM_OOCMF(alarm)	IS_TE_ALARM(alarm, WAN_TE_BIT_ALARM_OOCMF)
#define IS_TE_ALARM_OOOF(alarm)		IS_TE_ALARM(alarm, WAN_TE_BIT_ALARM_OOOF)
#define IS_TE_ALARM_RAI(alarm)		IS_TE_ALARM(alarm, WAN_TE_BIT_ALARM_RAI)
#define IS_TE_ALARM_YEL(alarm)		IS_TE_ALARM(alarm, WAN_TE_BIT_ALARM_YEL)

#define IS_TE_ALARM_OOFDL(alarm)	IS_TE_ALARM(alarm, WAN_TE_BIT_ALARM_OOFDL)
#define IS_TE_ALARM_RAIC(alarm)		IS_TE_ALARM(alarm, WAN_TE_BIT_ALARM_RAIC)
#define IS_TE_ALARM_OOFC(alarm)		IS_TE_ALARM(alarm, WAN_TE_BIT_ALARM_OOFC)

/* Needed for backward compatibility */
#ifndef IS_TE_ALOS_ALARM
#define IS_TE_ALOS_ALARM IS_TE_ALARM_ALOS
#endif

#ifndef IS_TE_LOS_ALARM
#define IS_TE_LOS_ALARM IS_TE_ALARM_LOS
#endif

#ifndef IS_TE_RED_ALARM
#define IS_TE_RED_ALARM IS_TE_ALARM_RED
#endif

#ifndef IS_TE_AIS_ALARM
#define IS_TE_AIS_ALARM IS_TE_ALARM_AIS
#endif


#ifndef IS_TE_RAI_ALARM
#define IS_TE_RAI_ALARM IS_TE_ALARM_RAI
#endif


#ifndef IS_TE_YEL_ALARM
#define IS_TE_YEL_ALARM IS_TE_ALARM_YEL
#endif

#ifndef IS_TE_OOF_ALARM
#define IS_TE_OOF_ALARM IS_TE_ALARM_OOF
#endif

/* Performance monitor counters bit mask */
#define WAN_TE_BIT_PMON_LCV		0x01	/* line code violation counter */
#define WAN_TE_BIT_PMON_BEE		0x02	/* bit errror event (T1) */
#define WAN_TE_BIT_PMON_OOF		0x04	/* frame out of sync counter */
#define WAN_TE_BIT_PMON_FEB		0x08	/* far end block counter */
#define WAN_TE_BIT_PMON_CRC4		0x10	/* crc4 error counter (E1) */
#define WAN_TE_BIT_PMON_FER		0x20	/* framing bit error (T1-pmc) */
#define WAN_TE_BIT_PMON_FAS		0x40	/* Frame Alginment signal (E1) */

/* T1/E1 statistics bit mask */
#define WAN_TE_STATS_BIT_ALARM		0x0001
#define WAN_TE_STATS_BIT_RXLEVEL	0x0002

/* For T1 only */
#define WAN_T1_LBO_NONE		0x00
#define WAN_T1_LBO_0_DB		0x01
#define WAN_T1_LBO_75_DB	0x02
#define WAN_T1_LBO_15_DB	0x03
#define WAN_T1_LBO_225_DB	0x04
#define WAN_T1_0_110		0x05
#define WAN_T1_110_220		0x06
#define WAN_T1_220_330		0x07
#define WAN_T1_330_440		0x08
#define WAN_T1_440_550		0x09
#define WAN_T1_550_660		0x0A
/* For E1 only */
#define WAN_E1_120		0x0B
#define WAN_E1_75		0x0C
/* T1/E1 8 ports */
#define WAN_T1_0_133		0x0D
#define WAN_T1_133_266		0x0E
#define WAN_T1_266_399		0x0F
#define WAN_T1_399_533		0x10
#define WAN_T1_533_655		0x11

/* T1/E1: Recever */
#define WAN_TE1_RX_SLEVEL_NONE		0
#define WAN_TE1_RX_SLEVEL_43_DB		430	/* 43 dB   E1, RMONEN=0 */
#define WAN_TE1_RX_SLEVEL_36_DB		360	/* 36 dB   T1, RMONEN=0 */
#define WAN_TE1_RX_SLEVEL_30_DB		300 /* 30 dB   RMONEN=0 | 1 */
#define WAN_TE1_RX_SLEVEL_225_DB	225	/* 22.5 dB RMONEN=1 */
#define WAN_TE1_RX_SLEVEL_18_DB		180	/* 18 dB   RMONEN=0 */
#define WAN_TE1_RX_SLEVEL_175_DB	175	/* 17.5 dB RMONEN=1 */
#define WAN_TE1_RX_SLEVEL_12_DB		120	/* 12 dB   RMONEN=0 | 1 */
#define WAN_TE1_RX_SLEVEL_DECODE(slevel)						\
		((slevel) == WAN_TE1_RX_SLEVEL_43_DB) ? "43dB" :		\
		((slevel) == WAN_TE1_RX_SLEVEL_36_DB) ? "36dB" :		\
		((slevel) == WAN_TE1_RX_SLEVEL_30_DB) ? "30dB" :		\
		((slevel) == WAN_TE1_RX_SLEVEL_225_DB) ? "22.5dB" :		\
		((slevel) == WAN_TE1_RX_SLEVEL_18_DB) ? "18dB" :		\
		((slevel) == WAN_TE1_RX_SLEVEL_175_DB) ? "17.5dB" :		\
		((slevel) == WAN_TE1_RX_SLEVEL_12_DB) ? "12dB" : "0dB"

/* For T1 only (long or short haul) */
#define WAN_T1_LONG_HAUL	0x01
#define WAN_T1_SHORT_HAUL	0x02

/* Line loopback modes */
#define WAN_TE1_LB_NONE			0x00
#define WAN_TE1_LINELB_MODE		0x01
#define WAN_TE1_PAYLB_MODE		0x02
#define WAN_TE1_DDLB_MODE		0x03
#define WAN_TE1_TX_LINELB_MODE	0x04
#define WAN_TE1_LIU_ALB_MODE	0x05
#define WAN_TE1_LIU_LLB_MODE	0x06
#define WAN_TE1_LIU_RLB_MODE	0x07
#define WAN_TE1_LIU_DLB_MODE	0x08
#define WAN_TE1_TX_PAYLB_MODE	0x09
#define WAN_TE1_PCLB_MODE		0x0A

#define WAN_TE1_LB_MODE_DECODE(mode)						\
		((mode) == WAN_TE1_LINELB_MODE) ? "Line/Remote Loopback" :	\
		((mode) == WAN_TE1_PAYLB_MODE) ? "Payload Loopback" :		\
		((mode) == WAN_TE1_DDLB_MODE) ? "Local Diagnostic Digital Loopback" :	\
		((mode) == WAN_TE1_TX_LINELB_MODE) ? "TX Line Loopback" :	\
		((mode) == WAN_TE1_TX_PAYLB_MODE)  ? "TX Payload Loopback" : \
		((mode) == WAN_TE1_LIU_ALB_MODE) ? "Analog LIU Loopback" :	\
		((mode) == WAN_TE1_LIU_LLB_MODE) ? "Local LIU Loopback" :	\
		((mode) == WAN_TE1_LIU_RLB_MODE) ? "Remote LIU Loopback" :	\
		((mode) == WAN_TE1_LIU_DLB_MODE) ? "Dual LIU Loopback" :	\
		((mode) == WAN_TE1_PCLB_MODE) ? "Per-channel Loopback" :	\
						"Unknown Loopback"

/* Line loopback activate/deactive modes */
#define WAN_TE1_LB_ENABLE	0x01
#define WAN_TE1_LB_DISABLE	0x02
#define WAN_TE1_LB_ACTION_DECODE(action)			\
		((action) == WAN_TE1_LB_ENABLE) ? "Enable" :	\
		((action) == WAN_TE1_LB_DISABLE) ? "Disable" :	\
						"Unknown"

/* T1/E1 front end Master clock source */
#define WAN_TE1_REFCLK_OSC	0x00
#define WAN_TE1_REFCLK_LINE1	0x01
#define WAN_TE1_REFCLK_LINE2	0x02
#define WAN_TE1_REFCLK_LINE3	0x03
#define WAN_TE1_REFCLK_LINE4	0x04

/* Loopback commands (T1.107-1995 p.44) */
#define WAN_T1_FDL_MSG_TIME		250	/* 250 us */
#define WAN_T1_ESF_LINELB_TX_CNT	10	
#define WAN_T1_D4_LINELB_TX_CNT		5	

#define RBOC_CODE_YEL		0x00
#define LINELB_ACTIVATE_CODE	0x07
#define LINELB_DEACTIVATE_CODE	0x1C
#define PAYLB_ACTIVATE_CODE	0x0A
#define PAYLB_DEACTIVATE_CODE	0x19
#define UNIVLB_DEACTIVATE_CODE	0x12

#define WAN_TE1_BOC_LB_CODE_DECODE(boc)						\
		((boc) == LINELB_ACTIVATE_CODE) ? "LINELB Activate"	:	\
		((boc) == LINELB_DEACTIVATE_CODE) ? "LINELB Deactivate"	:	\
		((boc) == PAYLB_ACTIVATE_CODE) ? "PAYLB Activate"	:	\
		((boc) == PAYLB_DEACTIVATE_CODE) ? "PAYLB Deactivate"	:	\
		((boc) == UNIVLB_DEACTIVATE_CODE) ? "Universal LB Deactivate"	:	\
							"Unsupported BOC"
 
/* Interrupt polling delay */
#define POLLING_TE1_TIMER		1	/* 1 sec (was 1000 ticks) */
#define WAN_T1_ALARM_THRESHOLD_LOF_ON		(3)	// must be 2-3s
#define WAN_T1_ALARM_THRESHOLD_LOF_OFF		(10)		// must be 10s
#define WAN_T1_ALARM_THRESHOLD_AIS_ON		(3)	// must be 2-3s
#define WAN_T1_ALARM_THRESHOLD_AIS_OFF		(10)		// must be 10s
#define WAN_T1_ALARM_THRESHOLD_LOS_ON		(3)	// must be 2-3s
#define WAN_T1_ALARM_THRESHOLD_LOS_OFF		(10)		// must be 10s

/* TE1 critical flag */
#define TE_TIMER_RUNNING 		0x01
#define TE_TIMER_KILL 			0x02
#define LINELB_WAITING			0x03
#define LINELB_CODE_BIT			0x04
#define LINELB_CHANNEL_BIT		0x05
#define TE_CONFIGURED			0x06
#define TE_TIMER_EVENT_PENDING 		0x07
#define TE_TIMER_EVENT_INPROGRESS 	0x08


/* TE1 sw irq types */
enum{
	WAN_TE1_SWIRQ_SUBTYPE_LINKDOWN = 1,
	WAN_TE1_SWIRQ_SUBTYPE_LINKREADY,
	WAN_TE1_SWIRQ_SUBTYPE_LINKCRIT,
	WAN_TE1_SWIRQ_SUBTYPE_LINKUP,

	WAN_TE1_SWIRQ_SUBTYPE_ALARM_ON,
	WAN_TE1_SWIRQ_SUBTYPE_ALARM_OFF,

};
#define WAN_TE1_SWIRQ_SUBTYPE_DECODE(subtype)						\
		((subtype) == WAN_TE1_SWIRQ_SUBTYPE_LINKDOWN) 	 ? "Down" :		\
		((subtype) == WAN_TE1_SWIRQ_SUBTYPE_LINKREADY) 	 ? "Ready" :		\
		((subtype) == WAN_TE1_SWIRQ_SUBTYPE_LINKCRIT) 	 ? "Crit" :		\
		((subtype) == WAN_TE1_SWIRQ_SUBTYPE_LINKUP) 	 ? "UP" :		\
		((subtype) == WAN_TE1_SWIRQ_SUBTYPE_ALARM_ON) 	 ? "activating" :	\
		((subtype) == WAN_TE1_SWIRQ_SUBTYPE_ALARM_OFF) 	 ? "deactivating" :	\
								"Unknown"

enum {
	WAN_TE1_SWIRQ_TYPE_NONE 	= 0,

	WAN_TE1_SWIRQ_TYPE_LINK,
	WAN_TE1_SWIRQ_TYPE_ALARM_AIS,
	WAN_TE1_SWIRQ_TYPE_ALARM_LOS,
	WAN_TE1_SWIRQ_TYPE_ALARM_LOF,

	WAN_TE1_SWIRQ_MAX
};
#define WAN_TE1_SWIRQ_TYPE_DECODE(type)						\
		((type) == WAN_TE1_SWIRQ_TYPE_LINK) 	 ? "T1/E1 Link" :	\
		((type) == WAN_TE1_SWIRQ_TYPE_ALARM_AIS) ? "T1/E1 Alarm (AIS)" :\
		((type) == WAN_TE1_SWIRQ_TYPE_ALARM_LOS) ? "T1/E1 Alarm (LOS)" :\
		((type) == WAN_TE1_SWIRQ_TYPE_ALARM_LOF) ? "T1/E1 Alarm (LOF)" :\
								"Unknown"
 

/* TE1 timer flags (polling) */
#define TE_LINELB_TIMER			0x01
#define TE_LINKDOWN_TIMER		0x02
#define TE_SET_INTR			0x03
#define TE_RBS_READ			0x04
#define TE_LINKUP_TIMER			0x05
#define TE_SET_RBS			0x06
#define TE_UPDATE_PMON			0x07
#define TE_SET_LB_MODE			0x08
#define TE_RBS_ENABLE			0x09
#define TE_RBS_DISABLE			0x0A
#define TE_POLL_CONFIG			0x0B
#define TE_POLL_READ			0x0C
#define TE_POLL_WRITE			0x0D
#define TE_POLL_CONFIG_VERIFY		0x0E
#define TE_LINKCRIT_TIMER		0x0F
#define WAN_TE_POLL_LINKREADY		0x10
#define WAN_TE_POLL_BERT		0x11
#define WAN_TE_POLL_ALARM_PENDING	0x12

/* TE1 T1/E1 interrupt setting delay */
#define INTR_TE1_TIMER		150	/* 50 ms */

/* T1/E1 RBS flags (bit-map) */
#define WAN_TE_RBS_NONE		0x00
#define WAN_TE_RBS_UPDATE	0x01
#define WAN_TE_RBS_REPORT	0x02


#define IS_T1_CARD(card)	IS_T1_FEMEDIA(&(card)->fe)
#define IS_E1_CARD(card)	IS_E1_FEMEDIA(&(card)->fe)
#define IS_TE1_CARD(card)	IS_TE1_FEMEDIA(&(card)->fe)

#define FE_LBO(fe_cfg)			(fe_cfg)->cfg.te_cfg.lbo	
#define FE_CLK(fe_cfg)			(fe_cfg)->cfg.te_cfg.te_clock	
#define FE_REFCLK(fe_cfg)		(fe_cfg)->cfg.te_cfg.te_ref_clock	
#define FE_HIMPEDANCE_MODE(fe_cfg)	(fe_cfg)->cfg.te_cfg.high_impedance_mode
#define FE_ACTIVE_CH(fe_cfg)		(fe_cfg)->cfg.te_cfg.active_ch
#define FE_SIG_MODE(fe_cfg)		(fe_cfg)->cfg.te_cfg.sig_mode
#define FE_RX_SLEVEL(fe_cfg)		(fe_cfg)->cfg.te_cfg.rx_slevel

#define GET_TE_START_CHANNEL(fe)			\
	(IS_T1_FEMEDIA(fe) ? 1 :			\
	 IS_E1_FEMEDIA(fe) ? 0 :0)

#define GET_TE_CHANNEL_RANGE(fe)				\
	(IS_T1_FEMEDIA(fe) ? NUM_OF_T1_CHANNELS :\
	 IS_E1_FEMEDIA(fe) ? NUM_OF_E1_CHANNELS :0)
			

#define WAN_TE_PRN_ALARM(alarm, bit)	((alarm) & (bit)) ? "ON" : "OFF"

#define WAN_TE_PRN_ALARM_ALOS(alarm)	WAN_TE_PRN_ALARM(alarm, WAN_TE_BIT_ALARM_ALOS)
#define WAN_TE_PRN_ALARM_LOS(alarm)	WAN_TE_PRN_ALARM(alarm, WAN_TE_BIT_ALARM_LOS)
#define WAN_TE_PRN_ALARM_OOF(alarm)	WAN_TE_PRN_ALARM(alarm, WAN_TE_BIT_ALARM_OOF)
#define WAN_TE_PRN_ALARM_LOF(alarm)	WAN_TE_PRN_ALARM(alarm, WAN_TE_BIT_ALARM_LOF)
#define WAN_TE_PRN_ALARM_RED(alarm)	WAN_TE_PRN_ALARM(alarm, WAN_TE_BIT_ALARM_RED)
#define WAN_TE_PRN_ALARM_AIS(alarm)	WAN_TE_PRN_ALARM(alarm, WAN_TE_BIT_ALARM_AIS)
#define WAN_TE_PRN_ALARM_OOSMF(alarm)	WAN_TE_PRN_ALARM(alarm, WAN_TE_BIT_ALARM_OOSMF)
#define WAN_TE_PRN_ALARM_OOCMF(alarm)	WAN_TE_PRN_ALARM(alarm, WAN_TE_BIT_ALARM_OOCMF)
#define WAN_TE_PRN_ALARM_OOOF(alarm)	WAN_TE_PRN_ALARM(alarm, WAN_TE_BIT_ALARM_OOOF)
#define WAN_TE_PRN_ALARM_RAI(alarm)	WAN_TE_PRN_ALARM(alarm, WAN_TE_BIT_ALARM_RAI)
#define WAN_TE_PRN_ALARM_YEL(alarm)	WAN_TE_PRN_ALARM(alarm, WAN_TE_BIT_ALARM_YEL)

#define WAN_TE_PRN_ALARM_LIU_SC(alarm)	WAN_TE_PRN_ALARM(alarm, WAN_TE_BIT_ALARM_LIU_SC)
#define WAN_TE_PRN_ALARM_LIU_OC(alarm)	WAN_TE_PRN_ALARM(alarm, WAN_TE_BIT_ALARM_LIU_OC)
#define WAN_TE_PRN_ALARM_LIU_LOS(alarm)	WAN_TE_PRN_ALARM(alarm, WAN_TE_BIT_ALARM_LIU_LOS)

#define TECLK_DECODE(fe_cfg)			\
	(FE_CLK(fe_cfg) == WAN_NORMAL_CLK) ? "Normal" :	\
	(FE_CLK(fe_cfg) == WAN_MASTER_CLK) ? "Master" : "Unknown"

#define LBO_DECODE(fe_cfg)				\
	(FE_LBO(fe_cfg) == WAN_T1_LBO_0_DB)	? "0db" :	\
	(FE_LBO(fe_cfg) == WAN_T1_LBO_75_DB)	? "7.5db" :	\
	(FE_LBO(fe_cfg) == WAN_T1_LBO_15_DB)	? "15dB" :	\
	(FE_LBO(fe_cfg) == WAN_T1_LBO_225_DB)	? "22.5dB" :	\
	(FE_LBO(fe_cfg) == WAN_T1_0_110)	? "0-110ft" :	\
	(FE_LBO(fe_cfg) == WAN_T1_110_220)	? "110-220ft" :	\
	(FE_LBO(fe_cfg) == WAN_T1_220_330)	? "220-330ft" :	\
	(FE_LBO(fe_cfg) == WAN_T1_330_440)	? "330-440ft" :	\
	(FE_LBO(fe_cfg) == WAN_T1_440_550)	? "440-550ft" :	\
	(FE_LBO(fe_cfg) == WAN_T1_550_660)	? "550-660ft" : \
	(FE_LBO(fe_cfg) == WAN_T1_0_133)	? "0-133ft"   :	\
	(FE_LBO(fe_cfg) == WAN_T1_133_266)	? "133-266ft" :	\
	(FE_LBO(fe_cfg) == WAN_T1_266_399)	? "266-399ft" :	\
	(FE_LBO(fe_cfg) == WAN_T1_399_533)	? "399-533ft" :	\
	(FE_LBO(fe_cfg) == WAN_T1_533_655)	? "5330-599ft":	\
	(FE_LBO(fe_cfg) == WAN_E1_120)		? "120OH"     :	\
	(FE_LBO(fe_cfg) == WAN_E1_75)		? "75OH"      :	\
							"Unknown"

/****************************************************************************/

/* T1/E1/Analog signalling insertion mode */
typedef enum _wan_fe_signaling_model {
	WAN_FE_SIG_NONE,
	WAN_FE_SIG_CCS,		/* T1/E1 CCS (d-chan/Clear Channel) - this is a default during installation */
	WAN_FE_SIG_CAS_RAW,	/* T1/E1/Analog - CAS, Raw Mode (no processing of the bits)*/
	WAN_FE_SIG_IDLE_TX,	/* T1/E1 Idle Channel. Not RBS/CAS and not Clear Channel. 
						 * The idle pattern will default to 0x17. */
	/* T1 */
	WAN_FE_SIG_EANDM_SYMMETRICAL,	
	WAN_FE_SIG_EANDM_ASYMMETRICAL,
	WAN_FE_SIG_EANDM_43801,
	/* E1 */
	WAN_FE_SIG_Q422,
	WAN_FE_SIG_ARGENTINA_R2, 
	WAN_FE_SIG_BRAZIL_R2
} wan_fe_signaling_model_t;


#define WAN_FE_SIG_MODEL_DECODE(fe_cfg)					\
	((FE_SIG_MODE(fe_cfg)) == WAN_FE_SIG_NONE) ? "WAN_FE_SIG_NONE" :	\
	((FE_SIG_MODE(fe_cfg)) == WAN_FE_SIG_CCS) ? "WAN_FE_SIG_CCS" :	\
	((FE_SIG_MODE(fe_cfg)) == WAN_FE_SIG_CAS_RAW) ? "WAN_FE_SIG_CAS_RAW" :	\
	((FE_SIG_MODE(fe_cfg)) == WAN_FE_SIG_IDLE_TX) ? "WAN_FE_SIG_IDLE_TX" :	\
	((FE_SIG_MODE(fe_cfg)) == WAN_FE_SIG_EANDM_SYMMETRICAL) ? "WAN_FE_SIG_EANDM_SYMMETRICAL" :	\
	((FE_SIG_MODE(fe_cfg)) == WAN_FE_SIG_EANDM_ASYMMETRICAL) ? "WAN_FE_SIG_EANDM_ASYMMETRICAL" :	\
	((FE_SIG_MODE(fe_cfg)) == WAN_FE_SIG_EANDM_43801) ? "WAN_FE_SIG_EANDM_43801" :	\
	((FE_SIG_MODE(fe_cfg)) == WAN_FE_SIG_Q422) ? "Q422" :	\
	((FE_SIG_MODE(fe_cfg)) == WAN_FE_SIG_ARGENTINA_R2) ? "WAN_FE_SIG_ARGENTINA_R2" :	\
	((FE_SIG_MODE(fe_cfg)) == WAN_FE_SIG_BRAZIL_R2) ? "WAN_FE_SIG_BRAZIL_R2" :	\
	"Unknown"

/******************************************
  Declarations for backward compatibility
*******************************************/
#define WAN_TE1_SIG_NONE	WAN_FE_SIG_NONE		/* default */
#define WAN_TE1_SIG_CCS		WAN_FE_SIG_CCS		/* E1 CCS - default */
#define WAN_TE1_SIG_CAS		WAN_FE_SIG_CAS_RAW	/* E1 CAS */


#define TE1SIG_DECODE	WAN_FE_SIG_MODEL_DECODE


/******************************************************
  RBS/CAS Signalling Bits for WAN_SIGNALING_MODEL_RAW 
*******************************************************/
#define WAN_RBS_SIG_A	0x01
#define WAN_RBS_SIG_B	0x02
#define WAN_RBS_SIG_C	0x04
#define WAN_RBS_SIG_D	0x08


/**************************************************
  RBS/CAS Signals.
  The Signals can be transmitted, received or both.
***************************************************/

enum _wan_rbs_cas_signals { 

	WAN_SIG_NONE,

	/* Callback notifications from API */
	WAN_SIG_TX_COMPLETE, /* Notifies about completion of signal transmission.
						  * Used for flow control of the TX signals. */
	WAN_SIG_TX_OVERFLOW, /* signal was dropped because stack is busy */
	WAN_SIG_TX_INVALID,	 /* Signal invalid for Model or simply invalid. */

	/* Signals common for many models */
	WAN_SIG_DIGIT,
	WAN_SIG_ONHOOK,
	WAN_SIG_OFFHOOK,
	WAN_SIG_WINK,
	WAN_SIG_FLASH,

	/* T1 */
	WAN_SIG_DS0AIS,
	WAN_SIG_DS0RAI,

	/* E&M43801 - specific */
	WAN_SIG_EM43801_LDOPEN,
	WAN_SIG_EM43801_LDCLSD,

	/* E1: Q422, Argentina R2, Brazil R2 */
	WAN_SIG_IDLE,
	WAN_SIG_SEIZE,
	WAN_SIG_SEIZEACK,
	WAN_SIG_ANSWER,
	WAN_SIG_CLEARFWD,
	WAN_SIG_CLEARBCK,
	WAN_SIG_BLOCKED,
    WAN_SIG_R2_METER,
    WAN_SIG_R2_FORCE
};

#if 0
	/* FXOGS */
    WAN_SIG_FXOGS_LO,
    WAN_SIG_FXOGS_LC,
    WAN_SIG_FXOGS_LCF,
    WAN_SIG_FXOGS_LCFO,
    WAN_SIG_FXOGS_RRING,
    WAN_SIG_FXOGS_RNGND
#endif

/* The definitions are NOT in wanpipe_events.h because it
 * will break kernel-independncy of FE Stack binaries. */

#define WAN_FE_SIGNAL_DECODE(signal)	\
	signal == WAN_SIG_NONE ? "WAN_SIG_NONE" :	\
	signal == WAN_SIG_TX_COMPLETE ? "WAN_SIG_TX_COMPLETE" :	\
	signal == WAN_SIG_TX_OVERFLOW ? "WAN_SIG_TX_OVERFLOW" :	\
	signal == WAN_SIG_TX_INVALID ? "WAN_SIG_TX_INVALID" :	\
	signal == WAN_SIG_DIGIT ? "WAN_SIG_DIGIT" :	\
	signal == WAN_SIG_ONHOOK ? "WAN_SIG_ONHOOK" :	\
	signal == WAN_SIG_OFFHOOK ? "WAN_SIG_OFFHOOK" :	\
	signal == WAN_SIG_WINK ? "WAN_SIG_WINK" :	\
	signal == WAN_SIG_FLASH ? "WAN_SIG_FLASH" :	\
	signal == WAN_SIG_DS0AIS ? "WAN_SIG_DS0AIS" :	\
	signal == WAN_SIG_DS0RAI ? "WAN_SIG_DS0RAI" :	\
	signal == WAN_SIG_EM43801_LDOPEN ? "WAN_SIG_EM43801_LDOPEN" :	\
	signal == WAN_SIG_EM43801_LDCLSD ? "WAN_SIG_EM43801_LDCLSD" :	\
	signal == WAN_SIG_IDLE ? "WAN_SIG_IDLE" :	\
	signal == WAN_SIG_SEIZE ? "WAN_SIG_SEIZE" :	\
	signal == WAN_SIG_SEIZEACK ? "WAN_SIG_SEIZEACK" :	\
	signal == WAN_SIG_ANSWER ? "WAN_SIG_ANSWER" :	\
	signal == WAN_SIG_CLEARFWD ? "WAN_SIG_CLEARFWD" :	\
	signal == WAN_SIG_CLEARBCK ? "WAN_SIG_CLEARBCK" :	\
	signal == WAN_SIG_BLOCKED ? "WAN_SIG_BLOCKED" :	\
	signal == WAN_SIG_R2_METER ? "WAN_SIG_R2_METER" :	\
	signal == WAN_SIG_R2_FORCE  ? "WAN_SIG_R2_FORCE" :	\
	"Unknown"

#define WAN_SIG_NO_DIGIT ((char)(-1))

/**************************************************/
	


/************************************
 event/command structure for RBS/CAS 
*************************************/

typedef struct _rbs_management{
	/* T1:1-24, E1:1-15,17-30 */
	int				channel;
	/* See _wan_rbs_cas_signals for list of values for 'signal'. 
	 * The 'signal' will be transmitted only if it is valid for
	 * the currently configured Signaling Model. */
	int				signal;
	/* The ABCD bits for RBS/CAS. It will be
	 * used when 'signal' is set to WAN_SIG_NONE. */
	unsigned char	ABCD_bits;
	/* Some of the non-RAW signaling models (E&M, Q422...)
	 * may transmit a digit.
	 * If 'signal' is set to WAN_SIG_DIGIT, the 'digit' will be
	 * transmitted (if Signaling Model allows digit transmission). */
	signed char		digit;
}rbs_management_t;


/****************************************************************************/

/* Front-End UDP command */
#define WAN_FE_SET_LB_MODE	WAN_FE_LB_MODE

#define WAN_FE_GET_STAT		(WAN_FE_UDP_CMD_START + 0)
#define WAN_FE_LB_MODE		(WAN_FE_UDP_CMD_START + 1)
#define WAN_FE_FLUSH_PMON	(WAN_FE_UDP_CMD_START + 2)
#define WAN_FE_GET_CFG		(WAN_FE_UDP_CMD_START + 3)
#define WAN_FE_SET_DEBUG_MODE	(WAN_FE_UDP_CMD_START + 4)
#define WAN_FE_TX_MODE		(WAN_FE_UDP_CMD_START + 5)
#define WAN_FE_BERT_MODE	(WAN_FE_UDP_CMD_START + 6)

/* FE interrupt types bit-map */
#define WAN_TE_INTR_NONE	0x00
#define WAN_TE_INTR_GLOBAL	0x01
#define WAN_TE_INTR_BASIC	0x02
#define WAN_TE_INTR_SIGNALLING	0x04
#define WAN_TE_INTR_FXS_DTMF	0x08
#define WAN_TE_INTR_PMON	0x10


/*----------------------------------------------------------------------------
 * T1/E1 configuration structures.
 */
typedef struct sdla_te_cfg {
	unsigned char	lbo;
	unsigned char	te_clock;
	unsigned int	active_ch;
	unsigned int	te_rbs_ch;
	unsigned char	high_impedance_mode; /* configure the chip in line 'Monitoring'
									  * aka 'Tapping' mode */
	int 		rx_slevel;	/* only for high_impedance_mode=YES */
	unsigned char	te_ref_clock;
	unsigned char	sig_mode;
	unsigned char	ignore_yel_alarm;
} sdla_te_cfg_t;

/****************************************/
/* T1/E1 Performamce monitor structures */
/****************************************/

typedef struct {

	/* valid intervals represented by a Summary Interval or
	 * total number of intervals when Current interval was retreived.
	 */
	int  total_intervals;
	
	unsigned long lastevent_elapsed_time;	/* seconds since last accum */

	/* timestamp 0 to 86399 seconds (maximum is 48 hours)
	 * The timestamp is the number of seconds since 
	 * midnight (T = 0) of the current day. There are
	 * 86400 seconds in 24 hours. The timestamp of any
	 * interval will always be between 0 and 85500
	 * (86400 - 900). Except for maybe the very first 
	 * interval, the timestamp represents the time at
	 * which that interval was started/created, which
	 * should always be on a 15-minute mark because
	 * that's when the intervals are pipelined, i.e. an increment of 900.
	 */
	long time_stamp;	

} sdla_te_pmon_interval_header_t;


/* Currently there is a single parameter in an interval.
 * More fields can be added in the future, hence the structure. */
typedef struct {
	long pm_value;
} sdla_te_pmon_interval_param_t;


/**************************************************/
/* Structures for T1.231 Performance Information. */
/**************************************************/
typedef struct {

	sdla_te_pmon_interval_header_t interval_header;
	/* Failure Count (Path).	FC-P.*/
	sdla_te_pmon_interval_param_t fc;
	/* Code Violation (Line).	CV-L */
	sdla_te_pmon_interval_param_t cvl;
	/* Errored Second (Line).	ES-L */
	sdla_te_pmon_interval_param_t esl;
	/* Severely Errored Second (Line). SES-L. */
	sdla_te_pmon_interval_param_t sesl;
	/* LOS Second (Line).		LOSS-L. */
	sdla_te_pmon_interval_param_t lossl;
	/* Code Violation (Path)	CV-P */
	sdla_te_pmon_interval_param_t cvp;
	/* Errored Second (Path). ES-P */
	sdla_te_pmon_interval_param_t esp;
	/* Errored Second type A (Path). ESA-P. */
	sdla_te_pmon_interval_param_t esap;
	/* Errored Second type B (Path). ESB-P. */
	sdla_te_pmon_interval_param_t esbp;
	/* Severly Errored Second (Path). SES-P. */
	sdla_te_pmon_interval_param_t sesp;
	/* SEF/AIS Second (Path). SAS-P. */
	sdla_te_pmon_interval_param_t sasp;
	/* AIS Second (Path). AISS-P. */
	sdla_te_pmon_interval_param_t aissp;
	/* Controlled Slip Second (Path). CSS-P. */
	sdla_te_pmon_interval_param_t cssp;
	/* UnAvailable Second (Path). UAS-P. */
	sdla_te_pmon_interval_param_t uasp;

} sdla_te_pmon_T1_T1231_interval_near_end_t;

typedef struct {

	sdla_te_pmon_interval_header_t interval_header;
	/* Failure Count (Path).	FC-PFE. */
	sdla_te_pmon_interval_param_t fc;
	/* Errored Second (Line).	ES-LFE. */
	sdla_te_pmon_interval_param_t esl;
	/* Code Violation (Path).	CV-PFE. */
	sdla_te_pmon_interval_param_t cvp;
	/* Severely Errored Frame Second (Path). SEFS-PFE. */
	sdla_te_pmon_interval_param_t sefsp;
	/* Errored Second (Path). ES-PFE. */
	sdla_te_pmon_interval_param_t esp;
	/* Errored Second type A (Path). ESA-PFE. */
	sdla_te_pmon_interval_param_t esap;
	/* Errored Second type B (Path). ESB-PFE. */
	sdla_te_pmon_interval_param_t esbp;
	/* Severly Errored Second (Path). SES-PFE. */
	sdla_te_pmon_interval_param_t sesp;
	/* Controlled Slip Second (Path). CSS-PFE. */
	sdla_te_pmon_interval_param_t cssp;
	/* UnAvailable Second (Path). UAS-PFE. */
	sdla_te_pmon_interval_param_t uasp;

} sdla_te_pmon_T1_T1231_interval_far_end_t;

/***************************************************/
/* Structures for E1 G826 Performance Information. */
/***************************************************/
typedef struct {

	sdla_te_pmon_interval_header_t interval_header;

	/* CV is not a part of G826 standard, added for convenience, 
	 * uses T1.231 definition. */
	sdla_te_pmon_interval_param_t cv;
	/* Controlled Slip Second (CSS/CS) */
	sdla_te_pmon_interval_param_t css;
	/* Out-Of-Frame (Seconds Counter) (OOF) */
	/* Not part of G826, but used to derive ES, SES,BBE (Annex B.1.3). */
	sdla_te_pmon_interval_param_t oof;
	/* Loss-Of-Signal (Seconds Counter) (LOS) */
	/* Not part of G826, but used to derive ES, SES,BBE (Annex B.1.3). */
	sdla_te_pmon_interval_param_t los;
	/* Alarm Indication Signal (Seconds Counter) (AIS) */
	/* Not part of G826, but used to derive ES, SES,BBE (Annex B.1.3). */
	sdla_te_pmon_interval_param_t ais;
	/* Remote Alarm Indication (Seconds Counter) (RAI) */
	/* Not part of G826, but used to derive ES, SES,BBE (Annex B.1.3). */
	sdla_te_pmon_interval_param_t rai;

	/* UnAvailable Seconds (UAS) */
	sdla_te_pmon_interval_param_t uas;
	/* Background Block Error Count (BBEC) */
	sdla_te_pmon_interval_param_t bbec;
	/* Background Block Error Ratio - the ratio of BBEC to total seconds. */
	sdla_te_pmon_interval_param_t bber;
	/* Errored Second Count (ESC) */
	sdla_te_pmon_interval_param_t esc;
	/* Errored Second Ratio - the ratio of ES to total seconds. */
	sdla_te_pmon_interval_param_t esr;
	/* Severely Errored Second Count */
	sdla_te_pmon_interval_param_t sesc;
	/* Severely Errored Second Ratio - the ratio of SESC to total seconds. */
	sdla_te_pmon_interval_param_t sesr;

} sdla_te_pmon_E1_G826_interval_near_end_t;

typedef struct {

	sdla_te_pmon_interval_header_t interval_header;
	/* Remote Alarm Indication (Signal) (RAI) */
	sdla_te_pmon_interval_param_t rai;
	/* UnAvailable Seconds (UAS) */
	sdla_te_pmon_interval_param_t uas;
	/* Background Block Error Count (BBEC) */
	sdla_te_pmon_interval_param_t bbec;
	/* Background Block Error Ratio - the ratio of BBEC to total seconds. */
	sdla_te_pmon_interval_param_t bber;
	/* Errored Second Count (ESC) */
	sdla_te_pmon_interval_param_t esc;
	/* Errored Second Ratio - the ratio of ES to total seconds. */
	sdla_te_pmon_interval_param_t esr;
	/* Severely Errored Second Count */
	sdla_te_pmon_interval_param_t sesc;
	/* Severely Errored Second Ratio - the ratio of SESC to total seconds. */
	sdla_te_pmon_interval_param_t sesr;

} sdla_te_pmon_E1_G826_interval_far_end_t;


typedef struct {
	/* T1/E1 - deprecated counters */
	unsigned short	mask;
	unsigned int	lcv_errors;	/* Line code violation (T1/E1) */
	unsigned short	lcv_diff;
	unsigned int	bee_errors;	/* Bit errors (T1) */
	unsigned short	bee_diff;
	unsigned int	oof_errors;	/* Frame out of sync (T1) */
	unsigned short	oof_diff;
	unsigned int	crc4_errors;	/* CRC4 errors (E1) */
	unsigned short	crc4_diff;
	unsigned int	fas_errors;	/* Frame Aligment Signal (E1)*/
	unsigned short	fas_diff;
	unsigned int	feb_errors;	/* Far End Block errors (E1) */
	unsigned short	feb_diff;
	unsigned int	fer_errors;	/* Framing bit errors (T1) */
	unsigned short	fer_diff;

	/* T1/E1 - current counters */
	union {
		/* T1 */
		sdla_te_pmon_T1_T1231_interval_near_end_t T1_T1231_near_end;
		sdla_te_pmon_T1_T1231_interval_far_end_t T1_T1231_far_end;
		/* E1 */
		sdla_te_pmon_E1_G826_interval_near_end_t E1_G826_near_end;
		sdla_te_pmon_E1_G826_interval_far_end_t E1_G826_far_end;
	};

} sdla_te_pmon_t;

/* Performance Monitor access macros */
#define WAN_PM_HEADER_VALUE(pm_header_struct, struct_member) \
	(pm_header_struct.struct_member)

#define WAN_PM_INTERVAL_VALUE(pm_interval_struct, counter_name)	\
	(pm_interval_struct.counter_name.pm_value)


typedef enum {

	WAN_PM_END_NEAR,
	WAN_PM_END_FAR

} wan_pm_end_t;

#define WAN_PM_END_DECODE(pm_end /* wan_pm_end_t */)	\
	(WAN_PM_END_NEAR == pm_end) ? "WAN_PM_END_NEAR" : \
	(WAN_PM_END_FAR == pm_end) ? "WAN_PM_END_FAR": \
	"Invalid PM End!"


typedef enum {

	/* There are 192 intervals, 15 minutes each (total 48 hours),
	 * plus the Current Interval. */
	WAN_PM_INTERVAL_CURRENT = 0,
	/* all values beween 0 and 192 are Indexes (do NOT change!!) */
	WAN_PM_INTERVAL_LAST = 192,
	/* the rest have special meaning */
	WAN_PM_INTERVAL_SUMMARY_TODAY,	/* summary after 00:00 (12 AM) up to Current */
	WAN_PM_INTERVAL_SUMMARY_YESTERDAY, /* summary before 00:00 (12 AM) */
	WAN_PM_INTERVAL_SUMMARY_24H /* summary of last 24 hours, up to Current */

} wan_pm_interval_t;



/* Structure for input parameters for 
 * WAN_FE_GET_STAT command. */
typedef struct {

	wan_pm_end_t pm_end;
	wan_pm_interval_t pm_interval;

} sdla_te_pmon_cmd_t;


#define WAN_TE_RXLEVEL_LEN	20
typedef struct {
	unsigned short	mask;
	sdla_te_pmon_t	pmon;
	char		rxlevel[WAN_TE_RXLEVEL_LEN];
} sdla_te_stats_t;

#define WAN_TE_BERT_CMD_NONE	0x00
#define WAN_TE_BERT_CMD_STOP	0x01
#define WAN_TE_BERT_CMD_START	0x02
#define WAN_TE_BERT_CMD_STATUS	0x03
#define WAN_TE_BERT_CMD_EIB	0x04
#define WAN_TE_BERT_CMD_RESET	0x05
#define WAN_TE_BERT_CMD_RUNNING	0x06
#define WAN_TE_BERT_CMD_DECODE(cmd)				\
	((cmd) == WAN_TE_BERT_CMD_STOP) ? "Stop" :	\
	((cmd) == WAN_TE_BERT_CMD_START) ? "Start" :		\
	((cmd) == WAN_TE_BERT_CMD_STATUS) ? "Status" :		\
	((cmd) == WAN_TE_BERT_CMD_EIB) ? "Error Insert Bit" : 	\
	((cmd) == WAN_TE_BERT_CMD_RESET) ? "Reset Status" : 	\
	((cmd) == WAN_TE_BERT_CMD_RUNNING) ? "Running" : 	\
						"Unknown"

/* BERT: Pattern type */
#define WAN_TE_BERT_PATTERN_PSEUDORANDOM_2E7	0x01
#define WAN_TE_BERT_PATTERN_PSEUDORANDOM_2E11	0x02
#define WAN_TE_BERT_PATTERN_PSEUDORANDOM_2E15	0x03
#define WAN_TE_BERT_PATTERN_PSEUDORANDOM_QRSS	0x04
#define WAN_TE_BERT_PATTERN_REPETITIVE		0x05
#define WAN_TE_BERT_PATTERN_WORD		0x06
#define WAN_TE_BERT_PATTERN_DALY		0x07
#define WAN_TE_BERT_PATTERN_PSEUDORANDOM_2E9	0x08
#define WAN_TE_BERT_PATTERN_DECODE(type)						\
	((type) == WAN_TE_BERT_PATTERN_PSEUDORANDOM_2E7) ? "PseudoRandom 2E7-1" :	\
	((type) == WAN_TE_BERT_PATTERN_PSEUDORANDOM_2E11) ? "PseudoRandom 2E11-1" :	\
	((type) == WAN_TE_BERT_PATTERN_PSEUDORANDOM_2E15) ? "PseudoRandom 2E15-1" :	\
	((type) == WAN_TE_BERT_PATTERN_PSEUDORANDOM_QRSS) ? "PseudoRandom QRSS" :	\
	((type) == WAN_TE_BERT_PATTERN_REPETITIVE) ? "Repetitive Pattern" :		\
	((type) == WAN_TE_BERT_PATTERN_WORD) ? "Alternating Word Pattern" :		\
	((type) == WAN_TE_BERT_PATTERN_DALY) ? "Daly Pattern" :			\
	((type) == WAN_TE_BERT_PATTERN_PSEUDORANDOM_2E9) ? "PseudoRandom 2E-9-1" : "Unknown"

#define WAN_TE_BERT_LOOPBACK_NONE		0x00
#define WAN_TE_BERT_LOOPBACK_PAYLOAD		0x01
#define WAN_TE_BERT_LOOPBACK_LINE		0x02
#define WAN_TE_BERT_LOOPBACK_DECODE(type)				\
	((type) == WAN_TE_BERT_LOOPBACK_NONE) ? "Loopback (None)" :	\
	((type) == WAN_TE_BERT_LOOPBACK_PAYLOAD) ? "Payload Loopback" :	\
	((type) == WAN_TE_BERT_LOOPBACK_LINE) ? "Line Loopback" :	\
						"Unknown"

/* BERT: Errot Insert Bits */
#define WAN_TE_BERT_EIB_NONE	0x00
#define WAN_TE_BERT_EIB_SINGLE	0x01
#define WAN_TE_BERT_EIB1	0x02
#define WAN_TE_BERT_EIB2	0x03
#define WAN_TE_BERT_EIB3	0x04
#define WAN_TE_BERT_EIB4	0x05
#define WAN_TE_BERT_EIB5	0x06
#define WAN_TE_BERT_EIB6	0x07
#define WAN_TE_BERT_EIB7	0x08
#define WAN_TE_BERT_EIB_DECODE(eib)					\
	((eib) == WAN_TE_BERT_EIB_NONE) ? "No error insertion" :	\
	((eib) == WAN_TE_BERT_EIB_SINGLE) ? "Single Bit Error Insert" :	\
	((eib) == WAN_TE_BERT_EIB1) ? "EIB 10E-1" :			\
	((eib) == WAN_TE_BERT_EIB2) ? "EIB 10E-2" :			\
	((eib) == WAN_TE_BERT_EIB3) ? "EIB 10E-3" :			\
	((eib) == WAN_TE_BERT_EIB4) ? "EIB 10E-4" :			\
	((eib) == WAN_TE_BERT_EIB5) ? "EIB 10E-5" :			\
	((eib) == WAN_TE_BERT_EIB6) ? "EIB 10E-6" :			\
	((eib) == WAN_TE_BERT_EIB7) ? "EIB 10E-7" : 			\
					"Unknown"

#define WAN_TE_BERT_STATUS_AUTO		0x00
#define WAN_TE_BERT_STATUS_MANUAL	0x01
#define WAN_TE_BERT_STATUS_RUNNING	0x02
#define WAN_TE_BERT_STATUS_STOPPED	0x03

#define WAN_TE_BERT_FLAG_VERBOSE	1
#define WAN_TE_BERT_FLAG_READY		2
#define WAN_TE_BERT_FLAG_INLOCK		3

#define WAN_TE_BERT_RC_SUCCESS		0
#define WAN_TE_BERT_RC_EINVAL		1
#define WAN_TE_BERT_RC_FAILED		2
#define WAN_TE_BERT_RC_RUNNING		3
#define WAN_TE_BERT_RC_STOPPED		4
#define WAN_TE_BERT_RC_DECODE(rc)					\
	((rc) == WAN_TE_BERT_RC_SUCCESS) ? "Success" :			\
	((rc) == WAN_TE_BERT_RC_EINVAL) ? "Invalid parameter" :		\
	((rc) == WAN_TE_BERT_RC_FAILED) ? "Failed" :			\
	((rc) == WAN_TE_BERT_RC_RUNNING) ? "BERT is still running" :	\
	((rc) == WAN_TE_BERT_RC_STOPPED) ? "BERT is not running" :	\
						"Unknown"

typedef struct 
{
	unsigned int	chan_map;	/* */
	unsigned short	pattern_type;	/* pattern type */
	unsigned short	pattern_len;	/* pattern length */
	unsigned int	pattern;	/* pattern */
	int		count;
	unsigned char	lb_type;
	unsigned short	eib;

} sdla_te_bert_cfg_t;

typedef struct 
{
	int		avail_sec;	/* out: */
	int		err_sec;	/* out: */
	int		err_free_sec;	/* out: */
	unsigned long	bit_cnt;	/* out: */
	unsigned long	err_cnt;	/* out: */	
	int		inlock;		/* out: */

} sdla_te_bert_stats_t;

typedef struct 
{
	unsigned int		chan_map;
	unsigned char		lb_type;

} sdla_te_bert_stop_t;

typedef struct 
{
	unsigned char	cmd;
	int		verbose;
	int		rc;
	unsigned char	status;

	union {
		sdla_te_bert_cfg_t	cfg;
		sdla_te_bert_stats_t	stats;
		sdla_te_bert_stop_t	stop;
	} un;

} sdla_te_bert_t;

/*
 ******************************************************************************
			STRUCTURES AND TYPEDEFS
 ******************************************************************************
*/

#ifdef WAN_KERNEL

/* Connection status threshold */
/* Original 5 
** Note (May 7 2009: We are waiting at least 10 sec anyway for other alarms */
#define WAN_TE1_STATUS_THRESHOLD	3	/* Feb 18, 2010
						 * DavidR: changed to 3 from 1 to
						 * avoid line bouncing during port
						 * startup. */

#define WAN_TE1_LBO(fe)		FE_LBO(&((fe)->fe_cfg))
#define WAN_TE1_CLK(fe)		FE_CLK(&((fe)->fe_cfg))
#define WAN_TE1_REFCLK(fe)	FE_REFCLK(&((fe)->fe_cfg))

/* May 4, 2010. DavidR: "high impedance" is misleading,
 *		often confused with Tx tri-state, which is a
 *		different setting (see FE_TXTRISTATE).
 *		What it actually does is controlling the Tapping mode. */
#define WAN_TE1_TAPPING_MODE(fe)	WAN_TE1_HI_MODE(fe)
/* All new code must use WAN_TE1_TAPPING_MODE(),
 * Keeping WAN_TE1_HI_MODE() for backwards compatibility. */
#define WAN_TE1_HI_MODE(fe)		FE_HIMPEDANCE_MODE(&((fe)->fe_cfg))

#define WAN_TE1_ACTIVE_CH(fe)	FE_ACTIVE_CH(&((fe)->fe_cfg))
#define WAN_TE1_SIG_MODE(fe)	FE_SIG_MODE(&((fe)->fe_cfg))

#define TE_LBO_DECODE(fe)	LBO_DECODE(&((fe)->fe_cfg))
#define TE_CLK_DECODE(fe)	TECLK_DECODE(&((fe)->fe_cfg))

#define WAN_TE1_SIG_DECODE(fe)	TE1SIG_DECODE(&((fe)->fe_cfg))


/* -----------------------------------------------------------------------------
 * Constants for the SET_T1_E1_SIGNALING_CFG/READ_T1_E1_SIGNALING_CFG commands
 * ---------------------------------------------------------------------------*/

/* the structure for setting the signaling permission */
#pragma pack(1)
typedef struct {
	unsigned char time_slot[32];
} te_signaling_perm_t;
#pragma pack()

/* settings for the signaling permission structure */
#define TE_SIG_DISABLED		0x00 /* signaling is disabled */
#define TE_RX_SIG_ENABLED	0x01 /* receive signaling is enabled */
#define TE_TX_SIG_ENABLED	0x02 /* transmit signaling is enabled */
#define TE_SET_TX_SIG_BITS	0x80 /* a flag indicating that outgoing 
					signaling bits should be set */

/* the structure used for the 
 * SET_T1_E1_SIGNALING_CFG/READ_T1_E1_SIGNALING_CFG command 
 */
#pragma pack(1)
typedef struct {
	/* signaling permission structure */
	te_signaling_perm_t sig_perm;
	/* loop signaling processing counter */
	unsigned char sig_processing_counter;
	/* pointer to the signaling permission structure */
	unsigned long ptr_te_sig_perm_struct;
	/* pointer to the receive signaling structure */
	unsigned long ptr_te_Rx_sig_struct;
	/* pointer to the transmit signaling structure */
	unsigned long ptr_te_Tx_sig_struct;
} te_signaling_cfg_t;
#pragma pack()

/* the structure used for reading and setting the signaling bits */
#pragma pack(1)
typedef struct {
	unsigned char time_slot[32];
} te_signaling_status_t;
#pragma pack()

typedef struct {
	unsigned long	ch_map;
	int 		rbs_channel;	/* tx rbs per channel */
	unsigned char	rbs_abcd;	/* tx rbs pec channel */
	unsigned char	lb_type;	/* loopback type */	
	unsigned short	reg;		/* fe register */	
	unsigned char	value;		/* fe register value */	
} sdla_te_event_t;

typedef struct {

	unsigned char	lb_tx_cmd;
	unsigned char	lb_tx_mode;
	unsigned char	lb_tx_code;
	unsigned long	lb_tx_cnt;

	unsigned long	critical;
	
	//unsigned char	timer_cmd;
	//unsigned long	timer_ch_map;
	//int 		timer_channel;	/* tx rbs per channel */
	//int		timer_delay;
	//unsigned char	timer_abcd;	/* tx rbs pec channel */
		
	unsigned char	SIGX_chg_30_25;
	unsigned char	SIGX_chg_24_17;
	unsigned char	SIGX_chg_16_9;
	unsigned char	SIGX_chg_8_1;

	unsigned long	ptr_te_sig_perm_off;
	unsigned long	ptr_te_Rx_sig_off;
	unsigned long	ptr_te_Tx_sig_off;

	unsigned char	intr_src1;
	unsigned char	intr_src2;
	unsigned char	intr_src3;

	unsigned int	max_channels;

	unsigned long	rx_rbs_status;
	unsigned char	rx_rbs[32];
	unsigned char	tx_rbs[32];

	unsigned long	tx_rbs_A;
	unsigned long	tx_rbs_B;
	unsigned long	tx_rbs_C;
	unsigned long	tx_rbs_D;

	unsigned long	rx_rbs_A;
	unsigned long	rx_rbs_B;
	unsigned long	rx_rbs_C;
	unsigned long	rx_rbs_D;

	unsigned char	xlpg_scale;
	
	unsigned short	status_cnt;

	int		reg_dbg_busy;	
	int		reg_dbg_ready;	
	unsigned char	reg_dbg_value;

	unsigned long	crit_alarm_start;
	unsigned int	lb_mode_map;
	
	int		tx_yel_alarm;

	unsigned int		bert_flag;
	unsigned long		bert_start;
	sdla_te_bert_cfg_t	bert_cfg;
	sdla_te_bert_stats_t	bert_stats;

#if 1 /*defined(AFT_NCOMM_TE1)*/
	unsigned int TMS_alarms_bitmap;
	unsigned char CAS_Signaling_Model; /* the default is Transparent Model - aka Raw RBS */
#endif
} sdla_te_param_t;



/*
 ******************************************************************************
			  FUNCTION PROTOTYPES
 ******************************************************************************
*/

WP_EXTERN int sdla_te_default_cfg(void* pfe, void* fe_cfg, int media);
WP_EXTERN int sdla_te_copycfg(void* pfe, void* fe_cfg);

WP_EXTERN int sdla_te_iface_init(void *p_fe, void *p_fe_iface);
WP_EXTERN int sdla_ds_te1_iface_init(void *p_fe, void *p_fe_iface);

WP_EXTERN int sdla_fe_stack_te1_iface_init(void *p_fe, void *p_fe_iface);

/*******************************************************
 * bit-definitions for 'sdlahw_info_t->fe_rw_bitmap'
 * bits 0-2 used by aft core without being defined here */
#define SDLA_FE_CPLD_REENTRY_BIT	(3)	/* FE and CPLD access must NOT
										 * interfere with each other! */
#define SDLA_FE_SHUTDOWN_BIT		(4)
/*******************************************************/

#endif /* WAN_KERNEL */

#undef WP_EXTERN

/* Deprecated defines */
#define WAN_TE_ALARM_MASK_FRAMER    	WAN_TE_ALARM_FRAMER_MASK 
#define WAN_TE_ALARM_MASK_LIU       	WAN_TE_ALARM_LIU_MASK 
#define WAN_TE_BIT_ALOS_ALARM         	WAN_TE_BIT_ALARM_ALOS 
#define WAN_TE_BIT_LOS_ALARM         	WAN_TE_BIT_ALARM_LOS 
#define WAN_TE_BIT_ALTLOS_ALARM         WAN_TE_BIT_ALARM_ALTLOS 
#define WAN_TE_BIT_OOF_ALARM         	WAN_TE_BIT_ALARM_OOF 
#define WAN_TE_BIT_RED_ALARM         	WAN_TE_BIT_ALARM_RED 
#define WAN_TE_BIT_AIS_ALARM         	WAN_TE_BIT_ALARM_AIS 
#define WAN_TE_BIT_OOSMF_ALARM         	WAN_TE_BIT_ALARM_OOSMF 
#define WAN_TE_BIT_OOCMF_ALARM         	WAN_TE_BIT_ALARM_OOCMF 
#define WAN_TE_BIT_OOOF_ALARM         	WAN_TE_BIT_ALARM_OOOF 
#define WAN_TE_BIT_RAI_ALARM         	WAN_TE_BIT_ALARM_RAI 
#define WAN_TE_BIT_YEL_ALARM         	WAN_TE_BIT_ALARM_YEL  
#define WAN_TE_BIT_LIU_ALARM         	WAN_TE_BIT_ALARM_LIU 
#define WAN_TE_BIT_LIU_ALARM_SC         WAN_TE_BIT_ALARM_LIU_SC 
#define WAN_TE_BIT_LIU_ALARM_OC         WAN_TE_BIT_ALARM_LIU_OC 
#define WAN_TE_BIT_LIU_ALARM_LOS     	WAN_TE_BIT_ALARM_LIU_LOS 

#define WAN_TE_ALARM(alarm, bit)    ((alarm) & (bit)) ? "ON" : "OFF"

#define WAN_TE_ALOS_ALARM(alarm)    	WAN_TE_ALARM(alarm, WAN_TE_BIT_ALOS_ALARM)
#define WAN_TE_LOS_ALARM(alarm)        	WAN_TE_ALARM(alarm, WAN_TE_BIT_LOS_ALARM)
#define WAN_TE_OOF_ALARM(alarm)        	WAN_TE_ALARM(alarm, WAN_TE_BIT_OOF_ALARM)
#define WAN_TE_RED_ALARM(alarm)        	WAN_TE_ALARM(alarm, WAN_TE_BIT_RED_ALARM)
#define WAN_TE_AIS_ALARM(alarm)        	WAN_TE_ALARM(alarm, WAN_TE_BIT_AIS_ALARM)
#define WAN_TE_OOSMF_ALARM(alarm)    	WAN_TE_ALARM(alarm, WAN_TE_BIT_OOSMF_ALARM)
#define WAN_TE_OOCMF_ALARM(alarm)    	WAN_TE_ALARM(alarm, WAN_TE_BIT_OOCMF_ALARM)
#define WAN_TE_OOOF_ALARM(alarm)    	WAN_TE_ALARM(alarm, WAN_TE_BIT_OOOF_ALARM)
#define WAN_TE_RAI_ALARM(alarm)        	WAN_TE_ALARM(alarm, WAN_TE_BIT_RAI_ALARM)
#define WAN_TE_YEL_ALARM(alarm)        	WAN_TE_ALARM(alarm, WAN_TE_BIT_YEL_ALARM)    
#define WAN_TE_LIU_ALARM_SC(alarm)    	WAN_TE_ALARM(alarm, WAN_TE_BIT_LIU_ALARM_SC)
#define WAN_TE_LIU_ALARM_OC(alarm)    	WAN_TE_ALARM(alarm, WAN_TE_BIT_LIU_ALARM_OC)
#define WAN_TE_LIU_ALARM_LOS(alarm)    	WAN_TE_ALARM(alarm, WAN_TE_BIT_LIU_ALARM_LOS)


#endif /* _SDLA_TE1_H */
